﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using AutomataTim.AutomataClasses;
using Microsoft.DirectX;


namespace AutomataTim
{
    public partial class AutomataDrawer : Control
    {
        public Automata Automata;
        public State CurrentState;     //selected
        public State ActiveState;        // current
        public Connection CurrentConnection;
        public PropertyGrid propertyGrid {get; set;}


        public AutomataDrawer()
        {
            InitializeComponent();
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
        }

        public void setActiveState(State state)
        {
            ActiveState = state;
        }

        public class drawpair : IEquatable<drawpair>
        {
            private int minid, maxid;
            public drawpair(int i1, int i2)
            {
                minid = Math.Min(i1, i2);
                maxid = Math.Max(i1, i2);
            }

            public override int GetHashCode()
            {
                //return base.GetHashCode();
                return minid * 1000 + maxid;
            }

            #region IEquatable<drawpair> Members

            public bool Equals(drawpair other)
            {
                if ((minid == other.minid) && (maxid == other.maxid))
                    return true;
                //if ((id1 == other.id2) && (id2 == other.id1))
                //    return true;
                return false;
                //throw new NotImplementedException();
            }

            #endregion
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            if (Automata != null)
            {
                pe.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                Pen grayPen = new Pen(Color.Gray, 1);
                Pen yellowPen = new Pen(Color.Orange, 3);
                grayPen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;
                yellowPen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;

              //  float offc = 0;
              //  int conindex = 0;

                Dictionary<String, Point> paintdata = new Dictionary<String, Point>();   // point as int stateID pair =))
                Dictionary<String, int> single = new Dictionary<String, int>();
              //  float step = 10.0f;    //curve offset (for multiple connections)
                int step = 10;
                

                foreach (Connection conn in Automata.Connections)
                {
                    State s1 = Automata.GetState(conn.State1Id);
                    State s2 = Automata.GetState(conn.State2Id);

                    if (s1 != null && s2 != null)
                    {
                        if (conn.State1Id == conn.State2Id)
                        {
                            String key = s1.Id.ToString();
                            if (single.ContainsKey(key))
                            {
                                single[key]++;
                            }
                            else
                            {
                                single.Add(key, 0);
                            }

                            Point pt1 = new Point(s1.Coordinates.X + 15, s1.Coordinates.Y + 15);
                            Point pt2 = new Point(pt1.X - 10 - single[key] * 6, pt1.Y + 25 + single[key] * 6);
                            Point pt3 = new Point(pt1.X + 10 + single[key] * 6, pt1.Y + 25 + single[key] * 6);
                            Point pt4 = new Point(s1.Coordinates.X + 15, s1.Coordinates.Y + 30);

                            Pen p = grayPen;
                            if (conn == CurrentConnection)
                                p = yellowPen;
                            
                            pe.Graphics.DrawCurve(p, new Point[] { pt1, pt2, pt3, pt4 }, 0.8f);
                        }

                        else
                        {
                            //AutomataDrawer.drawpair key = new AutomataDrawer.drawpair(s1.Id, s2.Id);      
                            String key = Math.Min(s1.Id, s2.Id).ToString() + " " + Math.Min(s1.Id, s2.Id).ToString();      
                            if (paintdata.ContainsKey(key))
                            {
                                paintdata[key] = new Point(-paintdata[key].X + (paintdata[key].Y % 2 == 0 ? step : 0), paintdata[key].Y + 1);
                            }
                            else
                            {
                                paintdata.Add(key, new Point(0, 0));  
                            }

                            Vector2 vect = new Vector2(s2.Coordinates.X - s1.Coordinates.X, s2.Coordinates.Y - s1.Coordinates.Y);
                            Vector2 normVect = vect, orthonormVect = vect;
                            normVect.Normalize();
                            orthonormVect.Normalize();
                            orthonormVect.TransformCoordinate(Matrix.RotationZ((float)(Math.PI / 2)));
                            if (orthonormVect.Y < 0)
                                orthonormVect *= -1f;

                            Point start, end;
                            //start = new Point(s1.Coordinates.X+(int)(normVect.X * 15)+15, s1.Coordinates.Y + (int)(normVect.Y * 15)+15);
                            Vector2 rotvect = normVect * 16;
                            
                            rotvect.TransformCoordinate(Matrix.RotationZ((float)(Math.PI / 90 * (double)paintdata[key].X)));
                            start = new Point(s1.Coordinates.X + (int)rotvect.X + 15, s1.Coordinates.Y + (int)rotvect.Y + 15);
                            //end = new Point(s2.Coordinates.X - (int)(normVect.X * 15)+15, s2.Coordinates.Y - (int)(normVect.Y * 15)+15);
                            end = new Point(s2.Coordinates.X - (int)rotvect.X + 15, s2.Coordinates.Y - (int)rotvect.Y + 15);
                            rotvect.TransformCoordinate(Matrix.RotationZ((float)(Math.PI)));
                            start = new Point(s1.Coordinates.X + (int)rotvect.X + 15, s1.Coordinates.Y + (int)rotvect.Y + 15);
                          
                            vect -= (normVect * 30);
                            vect *= 0.5f;
                            vect += (orthonormVect * (float)(paintdata[key].X));

                            Point pt1 = start;
                            Point pt2 = new Point(start.X + (int)vect.X, start.Y + (int)vect.Y);
                            Point pt3 = end;

                            Pen p = grayPen;
                            if (conn == CurrentConnection)
                                p = yellowPen;

                            pe.Graphics.DrawCurve(p, new Point[] { pt1, pt2, pt3 }, 0.7f);


                           


                            /*Point pt1 = new Point(s1.Coordinates.X + 15, s1.Coordinates.Y + 15);
                            Point pt2 = new Point(s2.Coordinates.X + 15, s2.Coordinates.Y + 15);

                            Vector2 vect = new Vector2(pt2.X - pt1.X, pt2.Y - pt1.Y);
                            Vector2 normVect = vect;
                            normVect.Normalize();
                            vect -= normVect * 15;

                            pt2.X = pt1.X + (int)vect.X;
                            pt2.Y = pt1.Y + (int)vect.Y;



                            Pen p = grayPen;
                            if (conn == CurrentConnection)
                                p = yellowPen;
                            pe.Graphics.DrawLine(p, pt1, pt2);*/
                        }
                    }
                }


                foreach (State s in Automata.States)
                {
                    string str = s.Id.ToString();
                    float height = pe.Graphics.MeasureString(str, SystemFonts.SmallCaptionFont).Height;
                    float width = pe.Graphics.MeasureString(str, SystemFonts.SmallCaptionFont).Width;
                    Brush brush = SystemBrushes.Info;
                    if (s == CurrentState)
                    {
                        brush = Brushes.GreenYellow;
                    }
                    if (s == Automata.curstate)
                    {
                        brush = Brushes.Blue;
                    }


                    pe.Graphics.FillEllipse(brush, s.Coordinates.X, s.Coordinates.Y, 30, 30);
                    pe.Graphics.DrawEllipse(SystemPens.ControlDarkDark, s.Coordinates.X, s.Coordinates.Y, 30, 30);
                    pe.Graphics.DrawString(s.Id.ToString(), SystemFonts.SmallCaptionFont, Brushes.Black, s.Coordinates.X+15 - width / 2, s.Coordinates.Y + 15 - height / 2);


                    
                }
            }
        }

        private bool moving = false;
        private int  connectionIndex = 0;
        private void AutomataDrawer_MouseDown(object sender, MouseEventArgs e)
        {
            Focus();
            if (Automata != null)
            {
                bool found = false;
                foreach (State s in Automata.States)
                {
                    Rectangle stateArea = new Rectangle(s.Coordinates.X, s.Coordinates.Y, 30, 30);
                    if (stateArea.Contains(e.Location))
                    {
                        if (CurrentState == s)
                        {
                            IEnumerable<Connection> connections = s.GetConnections();
                            if (connections.Count() > 0)
                            {
                                CurrentConnection = connections.ElementAt(connectionIndex++ % connections.Count());
                            }
                            else
                            {
                                CurrentConnection = null;
                            }
                            propertyGrid.SelectedObject = CurrentConnection;                            
                        }

                        else
                        {
                            connectionIndex = 0;
                            CurrentConnection = null;
                            propertyGrid.SelectedObject = CurrentConnection;
                        }
                        found = true;
                        CurrentState = s;
                        moving = true;
                        Invalidate();
                        break;
                    }
                }
                if (!found)
                {
                    CurrentState = null;
                    CurrentConnection = null;
                    propertyGrid.SelectedObject = CurrentConnection;
                }
            }
        }

        private void AutomataDrawer_MouseMove(object sender, MouseEventArgs e)
        {
            if (CurrentState != null && moving)
            {
                CurrentState.Coordinates.X = e.Location.X - 15;
                CurrentState.Coordinates.Y = e.Location.Y - 15;
                Invalidate();
            }
            
        }

        private void AutomataDrawer_MouseUp(object sender, MouseEventArgs e)
        {
            moving = false;
        }

        private void AutomataDrawer_MouseLeave(object sender, EventArgs e)
        {
            moving = false;
        }

        private void AutomataDrawer_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == ' ')
            {
                if (CurrentState != null)
                {
                    CurrentState.AddConnection(CurrentState);
                    Invalidate();
                }
            }

            if (char.ToLower(e.KeyChar) == 'a')
            {
                Automata.AddState();
                Invalidate();                
            }
            if (char.ToLower(e.KeyChar) == 'x')
                if (CurrentConnection != null)
                    Automata.Connections.Remove(CurrentConnection);

            if (char.ToLower(e.KeyChar) == 'd')
            {
                
       
                if (CurrentState != null)
                {
                    
                    Automata.DeleteState(CurrentState);
                    CurrentState = null;
                    CurrentConnection = null;
                    Invalidate();
                }
            }


        }

        
    }
}
